/*  OpenTally: Open-source election vote counting
 *  Copyright © 2021 Lee Yingtong Li (RunasSudo)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

mod fixed;
mod native;

#[cfg(not(target_arch = "wasm32"))]
mod rational_rug;

//#[cfg(target_arch = "wasm32")]
mod rational_num;

use num_traits::{NumAssignRef, NumRef};

use std::cmp::Ord;
use std::fmt;
use std::ops;

pub trait Assign<Src=Self> {
	fn assign(&mut self, src: Src);
}

pub trait From<T> {
	fn from(n: T) -> Self;
}

//pub trait Number: NumRef + NumAssignRef + PartialOrd + Assign + Clone + fmt::Display where for<'a> &'a Self: RefNum<&'a Self> {
pub trait Number:
	NumRef + NumAssignRef + ops::Neg<Output=Self> + Ord + Assign + From<usize> + From<f64> + Clone + fmt::Display
where
	for<'a> Self: Assign<&'a Self>
{
	fn new() -> Self;
	
	fn describe() -> String;
	fn describe_opt() -> String { Self::describe() }
	
	fn pow_assign(&mut self, exponent: i32);
	fn floor_mut(&mut self, dps: usize);
	fn ceil_mut(&mut self, dps: usize);
	
	fn parse(s: &str) -> Self {
		if let Ok(value) = Self::from_str_radix(s, 10) {
			return value;
		} else {
			panic!("Syntax Error");
		}
	}
}

pub use self::fixed::Fixed;
pub use self::native::NativeFloat64;

#[cfg(not(target_arch = "wasm32"))]
pub use self::rational_rug::Rational;

#[cfg(target_arch = "wasm32")]
pub use self::rational_num::Rational;
